@@ -1,6 +1,6 @@
 # -*- coding: utf-8; mode: tcl; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- vim:fenc=utf-8:filetype=tcl:et:sw=4:ts=4:sts=4
 # portconfigure.tcl
-# $Id: portconfigure.tcl 46624 2009-02-09 01:59:59Z ryandesign@macports.org $
+# $Id: portconfigure.tcl 55680 2009-08-17 03:53:54Z jmr@macports.org $
 #
 # Copyright (c) 2002 - 2003 Apple Computer, Inc.
 # Copyright (c) 2007 Markus W. Weissmann <mww@macports.org>
@@ -34,13 +34,16 @@
 package provide portconfigure 1.0
 package require portutil 1.0
 
-set org.macports.configure [target_new org.macports.configure configure_main]
+set org.macports.configure [target_new org.macports.configure portconfigure::configure_main]
 target_provides ${org.macports.configure} configure
 target_requires ${org.macports.configure} main fetch extract checksum patch
-target_prerun ${org.macports.configure} configure_start
+target_prerun ${org.macports.configure} portconfigure::configure_start
+
+namespace eval portconfigure {
+}
 
 # define options
-commands configure autoreconf automake autoconf xmkmf libtool
+commands configure autoreconf automake autoconf xmkmf
 # defaults
 default configure.env       ""
 default configure.pre_args  {--prefix=${prefix}}
@@ -54,6 +57,69 @@ default xmkmf.cmd           xmkmf
 default xmkmf.dir           {${worksrcpath}}
 default use_configure       yes
 
+option_proc use_autoreconf  portconfigure::set_configure_type
+option_proc use_automake    portconfigure::set_configure_type
+option_proc use_autoconf    portconfigure::set_configure_type
+option_proc use_xmkmf       portconfigure::set_configure_type
+
+option_proc autoreconf.cmd  portconfigure::set_configure_type
+option_proc automake.cmd    portconfigure::set_configure_type
+option_proc autoconf.cmd    portconfigure::set_configure_type
+option_proc xmkmf.cmd       portconfigure::set_configure_type
+
+##
+# Local helper proc
+proc portconfigure::add_build_dep { type dep } {
+    global ${type}.cmd option_defaults
+
+    if {![info exists ${type}.cmd] || (
+        ([info exists option_defaults(${type}.cmd)] && [set ${type}.cmd] == $option_defaults(${type}.cmd)) ||
+        (![info exists option_defaults(${type}.cmd)] && [set ${type}.cmd] == "${type}")
+        )} {
+            eval depends_build-append $dep
+    }
+}
+
+##
+# Adds dependencies for the binaries which will be called, but only if it is
+# the default. If .cmd was overwritten the port has to care for deps itself.
+proc portconfigure::set_configure_type {option action args} {
+    global autoreconf.cmd automake.cmd autoconf.cmd xmkmf.cmd
+
+    array set configure_map {
+        autoconf    {port:autoconf port:automake port:libtool}
+        xmkmf       port:imake
+    }
+
+    if {[string equal ${action} "set"]} {
+        switch $option {
+            autoreconf.cmd  -
+            automake.cmd    -
+            autoconf.cmd {
+                eval depends_build-delete $configure_map(autoconf)
+            }
+            xmkmf.cmd {
+                depends_build-delete $configure_map(xmkmf)
+            }
+            use_xmkmf {
+                if {[tbool args]} {
+                    depends_build-append $configure_map(xmkmf)
+                }
+            }
+            default {
+                # strip "use_"
+                set type [string range $option 4 end]
+                if {[tbool args]} {
+                    add_build_dep $type $configure_map(autoconf)
+                }
+            }
+        }
+    }
+}
+
+options configure.asroot
+default configure.asroot no
+
 # Configure special environment variables.
 # We could have m32/m64/march/mtune be global configurable at some point.
 options configure.m32 configure.m64 configure.march configure.mtune
@@ -63,36 +129,17 @@ default configure.mtune     {}
 options configure.optflags configure.cflags configure.cppflags configure.cxxflags configure.objcflags configure.ldflags configure.libs configure.fflags configure.f90flags configure.fcflags configure.classpath
 default configure.optflags  {-O2}
 # compiler flags section
-default configure.cflags    {[configure_get_cflags]}
-default configure.cppflags  {"-I${prefix}/include"}
-default configure.cxxflags  {[configure_get_cflags]}
-default configure.objcflags {[configure_get_cflags]}
-default configure.ldflags   {"-L${prefix}/lib"}
+default configure.cflags    {${configure.optflags}}
+default configure.cppflags  {-I${prefix}/include}
+default configure.cxxflags  {${configure.optflags}}
+default configure.objcflags {${configure.optflags}}
+default configure.ldflags   {-L${prefix}/lib}
 default configure.libs      {}
-default configure.fflags    {[configure_get_cflags]}
-default configure.f90flags  {[configure_get_cflags]}
-default configure.fcflags   {[configure_get_cflags]}
+default configure.fflags    {${configure.optflags}}
+default configure.f90flags  {${configure.optflags}}
+default configure.fcflags   {${configure.optflags}}
 default configure.classpath {}
 
-# internal function to return the system value for CFLAGS/CXXFLAGS/etc
-proc configure_get_cflags {args} {
-    global configure.optflags
-    global configure.m32 configure.m64 configure.march configure.mtune
-    set flags "${configure.optflags}"
-    if {[tbool configure.m64]} {
-        set flags "-m64 ${flags}"
-    } elseif {[tbool configure.m32]} {
-        set flags "-m32 ${flags}"
-    }
-    if {[info exists configure.march] && ${configure.march} != {}} {
-        set flags "${flags} -march=${configure.march}"
-    }
-    if {[info exists configure.mtune] && ${configure.mtune} != {}} {
-        set flags "${flags} -mtune=${configure.mtune}"
-    }
-    return $flags
-}
-
 # tools section
 options configure.perl configure.python configure.ruby configure.install configure.awk configure.bison configure.pkg_config configure.pkg_config_path
 default configure.perl              {}
@@ -104,52 +151,51 @@ default configure.bison             {}
 default configure.pkg_config        {}
 default configure.pkg_config_path   {}
 
-# backwards compatibility for hardcoded ports
-if {[file exists /Developer/SDKs/MacOSX10.5.sdk]} {
-    set sysroot "/Developer/SDKs/MacOSX10.5.sdk"
-} else {
-    set sysroot "/Developer/SDKs/MacOSX10.4u.sdk"
+options configure.build_arch
+default configure.build_arch {${build_arch}}
+foreach tool {cc cxx objc f77 f90 fc} {
+    options configure.${tool}_archflags
+    default configure.${tool}_archflags  "\[portconfigure::configure_get_archflags $tool\]"
 }
 
-options configure.universal_target configure.universal_sysroot configure.universal_archs configure.universal_args configure.universal_cflags configure.universal_cppflags configure.universal_cxxflags configure.universal_ldflags
-default configure.universal_target      {${universal_target}}
-default configure.universal_sysroot     {${universal_sysroot}}
+options configure.universal_archs configure.universal_args configure.universal_cflags configure.universal_cppflags configure.universal_cxxflags configure.universal_ldflags
 default configure.universal_archs       {${universal_archs}}
-default configure.universal_args        {[configure_get_universal_args]}
-default configure.universal_cflags      {[configure_get_universal_cflags]}
-default configure.universal_cppflags    {[configure_get_universal_cppflags]}
-default configure.universal_cxxflags    {[configure_get_universal_cflags]}
-default configure.universal_ldflags     {[configure_get_universal_ldflags]}
+default configure.universal_args        {--disable-dependency-tracking}
+default configure.universal_cflags      {[portconfigure::configure_get_universal_cflags]}
+default configure.universal_cppflags    {[portconfigure::configure_get_universal_cppflags]}
+default configure.universal_cxxflags    {[portconfigure::configure_get_universal_cflags]}
+default configure.universal_ldflags     {[portconfigure::configure_get_universal_ldflags]}
 
 # Select a distinct compiler (C, C preprocessor, C++)
 options configure.ccache configure.distcc configure.pipe configure.cc configure.cxx configure.cpp configure.objc configure.f77 configure.f90 configure.fc configure.javac configure.compiler
 default configure.ccache        {${configureccache}}
 default configure.distcc        {${configuredistcc}}
 default configure.pipe          {${configurepipe}}
-default configure.cc            {[configure_get_compiler cc]}
-default configure.cxx           {[configure_get_compiler cxx]}
-default configure.cpp           {[configure_get_compiler cpp]}
-default configure.objc          {[configure_get_compiler objc]}
-default configure.f77           {[configure_get_compiler f77]}
-default configure.f90           {[configure_get_compiler f90]}
-default configure.fc            {[configure_get_compiler fc]}
-default configure.javac         {[configure_get_compiler javac]}
-default configure.compiler      {[configure_get_default_compiler]}
+default configure.cc            {[portconfigure::configure_get_compiler cc]}
+default configure.cxx           {[portconfigure::configure_get_compiler cxx]}
+default configure.cpp           {[portconfigure::configure_get_compiler cpp]}
+default configure.objc          {[portconfigure::configure_get_compiler objc]}
+default configure.f77           {[portconfigure::configure_get_compiler f77]}
+default configure.f90           {[portconfigure::configure_get_compiler f90]}
+default configure.fc            {[portconfigure::configure_get_compiler fc]}
+default configure.javac         {[portconfigure::configure_get_compiler javac]}
+default configure.compiler      {[portconfigure::configure_get_default_compiler]}
 
 set_ui_prefix
 
-proc configure_start {args} {
-    global UI_PREFIX
-    global configure.compiler
+proc portconfigure::configure_start {args} {
+    global UI_PREFIX configure.compiler
     
-    ui_msg "$UI_PREFIX [format [msgcat::mc "Configuring %s"] [option portname]]"
+    ui_msg "$UI_PREFIX [format [msgcat::mc "Configuring %s"] [option name]]"
 
     set name ""
     switch -exact ${configure.compiler} {
+        gcc { set name "System gcc" }
         gcc-3.3 { set name "Mac OS X gcc 3.3" }
         gcc-4.0 { set name "Mac OS X gcc 4.0" }
         gcc-4.2 { set name "Mac OS X gcc 4.2" }
         llvm-gcc-4.2 { set name "Mac OS X llvm-gcc 4.2" }
+        clang { set name "Mac OS X clang" }
         apple-gcc-3.3 { set name "MacPorts Apple gcc 3.3" }
         apple-gcc-4.0 { set name "MacPorts Apple gcc 4.0" }
         apple-gcc-4.2 { set name "MacPorts Apple gcc 4.2" }
@@ -165,112 +211,118 @@ proc configure_start {args} {
     ui_debug "Using compiler '$name'"
 }
 
-# internal function to determine canonical system name for configure
-proc configure_get_universal_system_name {args} {
-    global configure.universal_target configure.universal_archs
-    set arch "unknown"
-    switch -- ${configure.universal_archs} {
-        "ppc"  { set arch "powerpc" }
-        "i386"  { set arch "i686" }
-        "ppc64"  { set arch "powerpc" }
-        "x86_64"  { set arch "i686" }
+# internal function to determine the compiler flags to select an arch
+proc portconfigure::configure_get_archflags {tool} {
+    global configure.build_arch configure.m32 configure.m64 configure.compiler
+    set flags ""
+    if {[tbool configure.m64]} {
+        set flags "-m64"
+    } elseif {[tbool configure.m32]} {
+        set flags "-m32"
+    } elseif {${configure.build_arch} != ""} {
+        if {[arch_flag_supported] && ($tool == "cc" || $tool == "cxx" || $tool == "objc")} {
+            set flags "-arch ${configure.build_arch}"
+        } elseif {${configure.build_arch} == "x86_64" || ${configure.build_arch} == "ppc64"} {
+            set flags "-m64"
+        } elseif {${configure.compiler} != "gcc-3.3"} {
+            set flags "-m32"
+        }
     }
-    switch -- ${configure.universal_target} {
-        "10.1"  { return "powerpc-apple-darwin5" }
-                # /Developer/SDKs/MacOSX10.1.5.sdk
-        "10.2"  { return "powerpc-apple-darwin6" }
-                # /Developer/SDKs/MacOSX10.2.8.sdk
-        "10.3"  { return "powerpc-apple-darwin7" }
-                # /Developer/SDKs/MacOSX10.3.9.sdk
-        "10.4"  { return "${arch}-apple-darwin8" }
-        "10.5"  { return "${arch}-apple-darwin9" }
-    }
-    return ""
-}
-
-# internal function to determine the universal args for configure.cmd
-proc configure_get_universal_args {args} {
-    global configure.universal_archs
-    set system [configure_get_universal_system_name]
-    set params "--disable-dependency-tracking"
-    if {[llength ${configure.universal_archs}] == 1 &&
-        [info exists system] && $system != ""} {
-        set params "$params --host=${system} --target=${system}"
-    }
-    return $params
+    return $flags
 }
 
 # internal function to determine the "-arch xy" flags for the compiler
-proc configure_get_universal_archflags {args} {
+proc portconfigure::configure_get_universal_archflags {args} {
     global configure.universal_archs
     set flags ""
     foreach arch ${configure.universal_archs} {
-        set flags "$flags -arch $arch"
+        if {$flags == ""} {
+            set flags "-arch $arch"
+        } else {
+            append flags " -arch $arch"
+        }
     }
     return $flags
 }
 
 # internal function to determine the CPPFLAGS for the compiler
-proc configure_get_universal_cppflags {args} {
-    global configure.universal_sysroot
+proc portconfigure::configure_get_universal_cppflags {args} {
+    global os.arch os.major developer_dir
     set flags ""
     # include sysroot in CPPFLAGS too (twice), for the benefit of autoconf
-    if {[info exists configure.universal_sysroot]} {
-        set flags "-isysroot ${configure.universal_sysroot}"
+    if {${os.arch} == "powerpc" && ${os.major} == "8"} {
+        set flags "-isysroot ${developer_dir}/SDKs/MacOSX10.4u.sdk"
     }
     return $flags
 }
 
 # internal function to determine the CFLAGS for the compiler
-proc configure_get_universal_cflags {args} {
-    global configure.universal_sysroot configure.universal_target
-    global os.platform os.arch os.version os.major
+proc portconfigure::configure_get_universal_cflags {args} {
+    global os.arch os.major developer_dir
     set flags [configure_get_universal_archflags]
     # these flags should be valid for C/C++ and similar compiler frontends
-    if {[info exists configure.universal_sysroot]} {
-        set flags "-isysroot ${configure.universal_sysroot} ${flags}"
-    }
-    # normally set in MACOSX_DEPLOYMENT_TARGET, add here too to make sure
-    if {${os.major} == "9"} {
-        set flags "${flags} -mmacosx-version-min=${configure.universal_target}"
+    if {${os.arch} == "powerpc" && ${os.major} == "8"} {
+        set flags "-isysroot ${developer_dir}/SDKs/MacOSX10.4u.sdk ${flags}"
     }
     return $flags
 }
 
 # internal function to determine the LDFLAGS for the compiler
-proc configure_get_universal_ldflags {args} {
-    global configure.universal_sysroot configure.universal_target
-    global os.platform os.arch os.version os.major
+proc portconfigure::configure_get_universal_ldflags {args} {
+    global os.arch os.major developer_dir
     set flags [configure_get_universal_archflags]
     # works around linking without using the CFLAGS, outside of automake
-    if {${os.arch} == "powerpc"} {
-        set flags "-Wl,-syslibroot,${configure.universal_sysroot} ${flags}"
-    }
-    # normally set in MACOSX_DEPLOYMENT_TARGET, add here too to make sure
-    if {${os.major} == "9"} {
-        set flags "${flags} -mmacosx-version-min=${configure.universal_target}"
+    if {${os.arch} == "powerpc" && ${os.major} == "8"} {
+        set flags "-Wl,-syslibroot,${developer_dir}/SDKs/MacOSX10.4u.sdk ${flags}"
     }
     return $flags
 }
 
+# internal proc to determine if the compiler supports -arch
+proc portconfigure::arch_flag_supported {args} {
+    global configure.compiler
+    switch -exact ${configure.compiler} {
+        gcc-4.0 -
+        gcc-4.2 -
+        llvm-gcc-4.2 -
+        clang -
+        apple-gcc-4.0 -
+        apple-gcc-4.2 {
+            return yes
+        }
+        default {
+            return no
+        }
+    }
+}
+
 # internal function to determine the default compiler
-proc configure_get_default_compiler {args} {
+proc portconfigure::configure_get_default_compiler {args} {
     global os.platform os.major
     set compiler ""
     switch -exact "${os.platform} ${os.major}" {
         "darwin 7" { set compiler gcc-3.3 }
         "darwin 8" { set compiler gcc-4.0 }
         "darwin 9" { set compiler gcc-4.0 }
-        "darwin 10" { set compiler llvm-gcc-4.2 }
+        "darwin 10" { set compiler gcc-4.2 }
+        default { set compiler gcc }
     }
     return $compiler
 }
 
 # internal function to find correct compilers
-proc configure_get_compiler {type} {
-    global configure.compiler prefix
+proc portconfigure::configure_get_compiler {type} {
+    global configure.compiler prefix developer_dir
     set ret ""
     switch -exact ${configure.compiler} {
+        gcc {
+            switch -exact ${type} {
+                cc   { set ret /usr/bin/gcc }
+                objc { set ret /usr/bin/gcc }
+                cxx  { set ret /usr/bin/g++ }
+                cpp  { set ret /usr/bin/cpp }
+            }
+        }
         gcc-3.3 {
             switch -exact ${type} {
                 cc   { set ret /usr/bin/gcc-3.3 }
@@ -297,10 +349,15 @@ proc configure_get_compiler {type} {
         }
         llvm-gcc-4.2 {
             switch -exact ${type} {
-                cc   { set ret /Developer/usr/llvm-gcc-4.2/bin/llvm-gcc-4.2 }
-                objc { set ret /Developer/usr/llvm-gcc-4.2/bin/llvm-gcc-4.2 }
-                cxx  { set ret /Developer/usr/llvm-gcc-4.2/bin/llvm-g++-4.2 }
-                cpp  { set ret /Developer/usr/llvm-gcc-4.2/bin/llvm-cpp-4.2 }
+                cc   { set ret ${developer_dir}/usr/llvm-gcc-4.2/bin/llvm-gcc-4.2 }
+                objc { set ret ${developer_dir}/usr/llvm-gcc-4.2/bin/llvm-gcc-4.2 }
+                cxx  { set ret ${developer_dir}/usr/llvm-gcc-4.2/bin/llvm-g++-4.2 }
+            }
+        }
+        clang {
+            switch -exact ${type} {
+                cc   { set ret ${developer_dir}/usr/bin/clang }
+                objc { set ret ${developer_dir}/usr/bin/clang }
             }
         }
         apple-gcc-3.3 {
@@ -396,36 +453,38 @@ proc configure_get_compiler {type} {
     return $ret
 }
 
-proc configure_main {args} {
+proc portconfigure::configure_main {args} {
     global [info globals]
     global worksrcpath use_configure use_autoreconf use_autoconf use_automake use_xmkmf
-    global configure.env configure.pipe configure.cflags configure.cppflags configure.cxxflags configure.objcflags configure.ldflags configure.libs configure.fflags configure.f90flags configure.fcflags configure.classpath
+    global configure.env configure.pipe configure.libs configure.classpath configure.universal_args
     global configure.perl configure.python configure.ruby configure.install configure.awk configure.bison configure.pkg_config configure.pkg_config_path
-    global configure.ccache configure.distcc configure.cc configure.cxx configure.cpp configure.objc configure.f77 configure.f90 configure.fc configure.javac
+    global configure.ccache configure.distcc configure.cpp configure.javac configure.march configure.mtune
+    foreach tool {cc cxx objc f77 f90 fc} {
+        global configure.${tool} configure.${tool}_archflags
+    }
+    foreach flags {cflags cppflags cxxflags objcflags ldflags fflags f90flags fcflags} {
+        global configure.${flags} configure.universal_${flags}
+    }
     
     if {[tbool use_autoreconf]} {
-        # XXX depend on autoreconf
         if {[catch {command_exec autoreconf} result]} {
             return -code error "[format [msgcat::mc "%s failure: %s"] autoreconf $result]"
         }
     }
     
     if {[tbool use_automake]} {
-        # XXX depend on automake
         if {[catch {command_exec automake} result]} {
             return -code error "[format [msgcat::mc "%s failure: %s"] automake $result]"
         }
     }
     
     if {[tbool use_autoconf]} {
-        # XXX depend on autoconf
         if {[catch {command_exec autoconf} result]} {
             return -code error "[format [msgcat::mc "%s failure: %s"] autoconf $result]"
         }
     }
 
     if {[tbool use_xmkmf]} {
-        # XXX depend on xmkmf
         if {[catch {command_exec xmkmf} result]} {
             return -code error "[format [msgcat::mc "%s failure: %s"] xmkmf $result]"
         } else {
@@ -458,7 +517,6 @@ proc configure_main {args} {
 
         # Append configure flags.
         append_list_to_environment_value configure "CC" ${filter}${configure.cc}
-        append_list_to_environment_value configure "CPP" ${filter}${configure.cpp}
         append_list_to_environment_value configure "CXX" ${filter}${configure.cxx}
         append_list_to_environment_value configure "OBJC" ${filter}${configure.objc}
         append_list_to_environment_value configure "FC" ${configure.fc}
@@ -483,6 +541,27 @@ proc configure_main {args} {
         append_list_to_environment_value configure "BISON" ${configure.bison}
         append_list_to_environment_value configure "PKG_CONFIG" ${configure.pkg_config}
         append_list_to_environment_value configure "PKG_CONFIG_PATH" ${configure.pkg_config_path}
+        
+        # add extra flags that are conditional on whether we're building universal
+        if {[variant_exists universal] && [variant_isset universal]} {
+            foreach flags {CFLAGS OBJCFLAGS} {
+                append_list_to_environment_value configure $flags ${configure.universal_cflags}
+            }
+            append_list_to_environment_value configure "CXXFLAGS" ${configure.universal_cxxflags}
+            append_list_to_environment_value configure "CPPFLAGS" ${configure.universal_cppflags}
+            append_list_to_environment_value configure "LDFLAGS" ${configure.universal_ldflags}
+            eval configure.pre_args-append ${configure.universal_args}
+        } else {
+            foreach {tool flags} {cc CFLAGS cxx CXXFLAGS objc OBJCFLAGS f77 FFLAGS f90 F90FLAGS fc FCFLAGS} {
+                append_list_to_environment_value configure $flags [set configure.${tool}_archflags]
+                if {${configure.march} != {}} {
+                    append_list_to_environment_value configure $flags "-march=${configure.march}"
+                }
+                if {${configure.mtune} != {}} {
+                    append_list_to_environment_value configure $flags "-mtune=${configure.mtune}"
+                }
+            }
+        }
 
         # Execute the command (with the new environment).
         if {[catch {command_exec configure} result]} {
