@@ -1,6 +1,6 @@
 # -*- coding: utf-8; mode: tcl; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- vim:fenc=utf-8:ft=tcl:et:sw=4:ts=4:sts=4
 # portfetch.tcl
-# $Id: portfetch.tcl 48236 2009-03-17 08:51:43Z blb@macports.org $
+# $Id: portfetch.tcl 57976 2009-09-19 22:19:57Z jmr@macports.org $
 #
 # Copyright (c) 2002 - 2003 Apple Computer, Inc.
 # All rights reserved.
@@ -16,7 +16,7 @@
 # 3. Neither the name of Apple Computer, Inc. nor the names of its contributors
 #    may be used to endorse or promote products derived from this software
 #    without specific prior written permission.
-# 
+#
 # THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 # AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 # IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -34,21 +34,35 @@ package provide portfetch 1.0
 package require portutil 1.0
 package require Pextlib 1.0
 
-set org.macports.fetch [target_new org.macports.fetch fetch_main]
-target_init ${org.macports.fetch} fetch_init
+set org.macports.fetch [target_new org.macports.fetch portfetch::fetch_main]
+target_init ${org.macports.fetch} portfetch::fetch_init
 target_provides ${org.macports.fetch} fetch
 target_requires ${org.macports.fetch} main
-target_prerun ${org.macports.fetch} fetch_start
+target_prerun ${org.macports.fetch} portfetch::fetch_start
+
+namespace eval portfetch {
+    namespace export suffix
+    variable fetch_urls {}
+    variable urlmap
+    array set urlmap {}
+}
+
+# Name space for internal site lists storage
+namespace eval portfetch::mirror_sites {
+    variable sites
+
+    array set sites {}
+}
 
 # define options: distname master_sites
 options master_sites patch_sites extract.suffix distfiles patchfiles use_zip use_bzip2 use_lzma use_7z use_dmg dist_subdir \
-	fetch.type fetch.user fetch.password fetch.use_epsv fetch.ignore_sslcert \
-	master_sites.mirror_subdir patch_sites.mirror_subdir portname \
-	cvs.module cvs.root cvs.password cvs.date cvs.tag \
-	svn.url svn.tag \
-	git.url git.branch \
-	hg.url hg.tag
-	
+    fetch.type fetch.user fetch.password fetch.use_epsv fetch.ignore_sslcert \
+    master_sites.mirror_subdir patch_sites.mirror_subdir \
+    cvs.module cvs.root cvs.password cvs.date cvs.tag cvs.method \
+    svn.url svn.revision svn.method \
+    git.cmd git.url git.branch \
+    hg.cmd hg.url hg.tag
+
 # XXX we use the command framework to buy us some useful features,
 # but this is not a user-modifiable command
 commands cvs
@@ -58,9 +72,10 @@ commands svn
 default extract.suffix .tar.gz
 default fetch.type standard
 
-default cvs.cmd {$portutil::autoconf::cvs_path}
+default cvs.cmd {[findBinary cvs $portutil::autoconf::cvs_path]}
 default cvs.password ""
 default cvs.dir {${workpath}}
+default cvs.method {export}
 default cvs.module {$distname}
 default cvs.tag ""
 default cvs.date ""
@@ -69,23 +84,27 @@ default cvs.pre_args {"-z9 -f -d ${cvs.root}"}
 default cvs.args ""
 default cvs.post_args {"${cvs.module}"}
 
-default svn.cmd {$portutil::autoconf::svn_path}
+# we want the svn port so we know --trust-server-cert will work
+default svn.cmd {[portfetch::find_svn_path]}
 default svn.dir {${workpath}}
-default svn.tag ""
+default svn.method {export}
+default svn.revision ""
 default svn.env {}
-default svn.pre_args {"--non-interactive"}
+default svn.pre_args {"--non-interactive --trust-server-cert"}
 default svn.args ""
 default svn.post_args {"${svn.url}"}
 
+default git.cmd {[findBinary git $portutil::autoconf::git_path]}
 default git.dir {${workpath}}
 default git.branch {}
 
+default hg.cmd {[findBinary hg $portutil::autoconf::hg_path]}
 default hg.dir {${workpath}}
 default hg.tag {tip}
 
 # Set distfiles
-default distfiles {[suffix $distname]}
-default dist_subdir {${portname}}
+default distfiles {[portfetch::suffix $distname]}
+default dist_subdir {${name}}
 
 # user name & password
 default fetch.user ""
@@ -102,14 +121,19 @@ default global_mirror_site "macports_distfiles"
 default mirror_sites.listfile {"mirror_sites.tcl"}
 default mirror_sites.listpath {"port1.0/fetch"}
 
-# Option-executed procedures
-option_proc use_bzip2 fix_extract_suffix
-option_proc use_lzma fix_extract_suffix
-option_proc use_zip fix_extract_suffix
-option_proc use_7z fix_extract_suffix
-option_proc use_dmg fix_extract_suffix
+# Deprecation
+option_deprecate svn.tag svn.revision
 
-proc fix_extract_suffix {option action args} {
+# Option-executed procedures
+option_proc use_bzip2 portfetch::set_extract_type
+option_proc use_lzma  portfetch::set_extract_type
+option_proc use_zip   portfetch::set_extract_type
+option_proc use_7z    portfetch::set_extract_type
+option_proc use_dmg   portfetch::set_extract_type
+
+option_proc fetch.type portfetch::set_fetch_type
+
+proc portfetch::set_extract_type {option action args} {
     global extract.suffix
     if {[string equal ${action} "set"] && [tbool args]} {
         switch $option {
@@ -118,12 +142,15 @@ proc fix_extract_suffix {option action args} {
             }
             use_lzma {
                 set extract.suffix .tar.lzma
+                depends_extract-append bin:lzma:lzmautils
             }
             use_zip {
                 set extract.suffix .zip
+                depends_extract-append bin:unzip:unzip
             }
             use_7z {
                 set extract.suffix .7z
+                depends_extract-append bin:7za:p7zip
             }
             use_dmg {
                 set extract.suffix .dmg
@@ -132,24 +159,57 @@ proc fix_extract_suffix {option action args} {
     }
 }
 
-# Name space for internal implementation variables
-# Site lists are stored here
-namespace eval portfetch { }
+proc portfetch::set_fetch_type {option action args} {
+    global os.platform os.major
+    if {[string equal ${action} "set"]} {
+        switch $args {
+            cvs {
+                depends_fetch-append bin:cvs:cvs
+            }
+            svn {
+                if {${os.platform} == "darwin" && ${os.major} >= 10} {
+                    depends_fetch-append bin:svn:subversion
+                } else {
+                    depends_fetch-append port:subversion
+                }
+            }
+            git {
+                depends_fetch-append bin:git:git-core
+            }
+            hg {
+                depends_fetch-append bin:hg:mercurial
+            }
+        }
+    }
+}
+
+proc portfetch::find_svn_path {args} {
+    global prefix os.platform os.major
+    if {${os.platform} == "darwin" && ${os.major} >= 10} {
+        return [findBinary svn $portutil::autoconf::svn_path]
+    } else {
+        return ${prefix}/bin/svn
+    }
+}
 
 set_ui_prefix
 
+
 # Given a distname, return a suffix based on the use_zip / use_bzip2 / use_dmg / extract.suffix options
-proc suffix {distname} {
+proc portfetch::suffix {distname} {
     global extract.suffix fetch.type
     switch -- "${fetch.type}" {
-    	cvs			-
-    	svn			-
-    	git			-
-    	hg			{ return "" }
-    	standard	-
-    	default 	{ return "${distname}${extract.suffix}" }
+        cvs         -
+        svn         -
+        git         -
+        hg          { return "" }
+        standard    -
+        default     { return "${distname}${extract.suffix}" }
     }
 }
+# XXX import suffix into the global namespace as it is currently used from
+# Portfiles, but should better go somewhere else
+namespace import portfetch::suffix
 
 # Given a site url and the name of the distfile, assemble url and
 # return it.
@@ -161,69 +221,63 @@ proc portfetch::assemble_url {site distfile} {
     }
 }
 
-# XXX
-# Helper function for portextract.tcl that strips all tag names from a list
-# Used to clean ${distfiles} for setting the ${extract.only} default
-proc disttagclean {list} {
-    if {"$list" == ""} {
-        return $list
-    }
-    foreach name $list {
-        lappend val [getdistname $name]
-    }
-    return $val
-}
-
 # For a given mirror site type, e.g. "gnu" or "x11", check to see if there's a
 # pre-registered set of sites, and if so, return them.
-proc mirror_sites {mirrors tag subdir} {
-    global UI_PREFIX portname porturl mirror_sites.listfile mirror_sites.listpath dist_subdir
+proc portfetch::mirror_sites {mirrors tag subdir} {
+    global UI_PREFIX name porturl mirror_sites.listfile mirror_sites.listpath dist_subdir
+    global global_mirror_site fallback_mirror_site
 
-    source [getportresourcepath $porturl [file join ${mirror_sites.listpath} ${mirror_sites.listfile}]]
+    set mirrorfile [getportresourcepath $porturl [file join ${mirror_sites.listpath} ${mirror_sites.listfile}]]
+    if {[file exists $mirrorfile]} {
+        source $mirrorfile
+    }
 
     if {![info exists portfetch::mirror_sites::sites($mirrors)]} {
-        ui_warn "[format [msgcat::mc "No mirror sites on file for class %s"] $mirrors]"
+        if {$mirrors != $global_mirror_site && $mirrors != $fallback_mirror_site} {
+            ui_warn "[format [msgcat::mc "No mirror sites on file for class %s"] $mirrors]"
+        }
         return {}
     }
-    
+
     set ret [list]
     foreach element $portfetch::mirror_sites::sites($mirrors) {
-	
-	# here we have the chance to take a look at tags, that possibly
-	# have been assigned in mirror_sites.tcl
-	set splitlist [split $element :]
-	# every element is a URL, so we'll always have multiple elements. no need to check
-    set element "[lindex $splitlist 0]:[lindex $splitlist 1]" 
-    set mirror_tag "[lindex $splitlist 2]"
 
-    set name_re {\$(?:name\y|\{name\})}
-    # if the URL has $name embedded, kill any mirror_tag that may have been added
-    # since a mirror_tag and $name are incompatible
-    if {[regexp $name_re $element]} {
-        set mirror_tag ""
-    }
-    
-	if {$mirror_tag == "mirror"} {
-		set thesubdir ${dist_subdir}
-	} elseif {$subdir == "" && $mirror_tag != "nosubdir"} {
-		set thesubdir ${portname}
-	} else {
-		set thesubdir ${subdir}
-	}
-	
-	# parse an embedded $name. if present, remove the subdir
-	if {[regsub $name_re $element $thesubdir element] > 0} {
-	    set thesubdir ""
-	}
-	
-	if {"$tag" != ""} {
-	    eval append element "${thesubdir}:${tag}"
-	} else {
-	    eval append element "${thesubdir}"
-	}
+        # here we have the chance to take a look at tags, that possibly
+        # have been assigned in mirror_sites.tcl
+        set splitlist [split $element :]
+        # every element is a URL, so we'll always have multiple elements. no need to check
+        set element "[lindex $splitlist 0]:[lindex $splitlist 1]"
+        set mirror_tag "[lindex $splitlist 2]"
+
+        set name_re {\$(?:name\y|\{name\})}
+        # if the URL has $name embedded, kill any mirror_tag that may have been added
+        # since a mirror_tag and $name are incompatible
+        if {[regexp $name_re $element]} {
+            set mirror_tag ""
+        }
+
+        if {$mirror_tag == "mirror"} {
+            set thesubdir ${dist_subdir}
+        } elseif {$subdir == "" && $mirror_tag != "nosubdir"} {
+            set thesubdir ${name}
+        } else {
+            set thesubdir ${subdir}
+        }
+
+        # parse an embedded $name. if present, remove the subdir
+        if {[regsub $name_re $element $thesubdir element] > 0} {
+            set thesubdir ""
+        }
+
+        if {"$tag" != ""} {
+            eval append element "${thesubdir}:${tag}"
+        } else {
+            eval append element "${thesubdir}"
+        }
+
         eval lappend ret $element
     }
-    
+
     return $ret
 }
 
@@ -232,18 +286,19 @@ proc mirror_sites {mirrors tag subdir} {
 # within that tag distfiles are added in $site $distfile format, where $site is
 # the name of a variable in the portfetch:: namespace containing a list of fetch
 # sites
-proc checksites {args} {
+proc portfetch::checksites {args} {
     global patch_sites master_sites master_sites.mirror_subdir \
         patch_sites.mirror_subdir fallback_mirror_site global_mirror_site env
-    
+    variable urlmap
+
     append master_sites " ${global_mirror_site} ${fallback_mirror_site}"
     if {[info exists env(MASTER_SITE_LOCAL)]} {
-	set master_sites [concat $env(MASTER_SITE_LOCAL) $master_sites]
+    set master_sites [concat $env(MASTER_SITE_LOCAL) $master_sites]
     }
-    
+
     append patch_sites " ${global_mirror_site} ${fallback_mirror_site}"
     if {[info exists env(PATCH_SITE_LOCAL)]} {
-	set patch_sites [concat $env(PATCH_SITE_LOCAL) $patch_sites]
+    set patch_sites [concat $env(PATCH_SITE_LOCAL) $patch_sites]
     }
 
     foreach list {master_sites patch_sites} {
@@ -251,26 +306,26 @@ proc checksites {args} {
         if {![info exists uplist]} {
             continue
         }
-        
+
         set site_list [list]
         foreach site $uplist {
             if {[regexp {([a-zA-Z]+://.+)} $site match site]} {
                 set site_list [concat $site_list $site]
             } else {
-	    	set splitlist [split $site :]
-		if {[llength $splitlist] > 3 || [llength $splitlist] <1} {
+            set splitlist [split $site :]
+        if {[llength $splitlist] > 3 || [llength $splitlist] <1} {
                     ui_error [format [msgcat::mc "Unable to process mirror sites for: %s, ignoring."] $site]
-		}
-		set mirrors "[lindex $splitlist 0]"
-		set subdir "[lindex $splitlist 1]"
-		set tag "[lindex $splitlist 2]"
+        }
+        set mirrors "[lindex $splitlist 0]"
+        set subdir "[lindex $splitlist 1]"
+        set tag "[lindex $splitlist 2]"
                 if {[info exists $list.mirror_subdir]} {
                     append subdir "[set ${list}.mirror_subdir]"
                 }
                 set site_list [concat $site_list [mirror_sites $mirrors $tag $subdir]]
             }
         }
-        
+
         # add in the global and fallback mirrors for each tag
         foreach site $site_list {
             if {[regexp {([a-zA-Z]+://.+/?):([0-9A-Za-z_-]+)$} $site match site tag]} {
@@ -285,73 +340,75 @@ proc checksites {args} {
                 }
             }
         }
-        
+
         foreach site $site_list {
-	    if {[regexp {([a-zA-Z]+://.+/?):([0-9A-Za-z_-]+)$} $site match site tag]} {
-                lappend portfetch::$tag $site
+        if {[regexp {([a-zA-Z]+://.+/?):([0-9A-Za-z_-]+)$} $site match site tag]} {
+                lappend urlmap($tag) $site
             } else {
-                lappend portfetch::$list $site
+                lappend urlmap($list) $site
             }
         }
     }
 }
 
 # Checks patch files and their tags to assemble url lists for later fetching
-proc checkpatchfiles {args} {
-    global patchfiles all_dist_files patch_sites fetch_urls filespath
-    
+proc portfetch::checkpatchfiles {urls} {
+    global patchfiles all_dist_files patch_sites filespath
+    upvar $urls fetch_urls
+
     if {[info exists patchfiles]} {
-	foreach file $patchfiles {
-	    if {![file exists $filespath/$file]} {
-		set distsite [getdisttag $file]
-		set file [getdistname $file]
-		lappend all_dist_files $file
-		if {$distsite != ""} {
-		    lappend fetch_urls $distsite $file
-		} elseif {[info exists patch_sites]} {
-		    lappend fetch_urls patch_sites $file
-		} else {
-		    lappend fetch_urls master_sites $file
-		}
-	    }
-	}
+        foreach file $patchfiles {
+            if {![file exists $filespath/$file]} {
+                set distsite [getdisttag $file]
+                set file [getdistname $file]
+                lappend all_dist_files $file
+                if {$distsite != ""} {
+                    lappend fetch_urls $distsite $file
+                } elseif {[info exists patch_sites]} {
+                    lappend fetch_urls patch_sites $file
+                } else {
+                    lappend fetch_urls master_sites $file
+                }
+            }
+        }
     }
 }
 
 # Checks dist files and their tags to assemble url lists for later fetching
-proc checkdistfiles {args} {
-    global distfiles all_dist_files fetch_urls master_sites filespath
-    
+proc portfetch::checkdistfiles {urls} {
+    global distfiles all_dist_files master_sites filespath
+    upvar $urls fetch_urls
+
     if {[info exists distfiles]} {
-    foreach file $distfiles {
-	if {![file exists $filespath/$file]} {
-	    set distsite [getdisttag $file]
-	    set file [getdistname $file]
-	    lappend all_dist_files $file
-	    if {$distsite != ""} {
-		lappend fetch_urls $distsite $file
-	    } else {
-		lappend fetch_urls master_sites $file
-	    }
-	}
-    }
+        foreach file $distfiles {
+            if {![file exists $filespath/$file]} {
+                set distsite [getdisttag $file]
+                set file [getdistname $file]
+                lappend all_dist_files $file
+                if {$distsite != ""} {
+                    lappend fetch_urls $distsite $file
+                } else {
+                    lappend fetch_urls master_sites $file
+                }
+            }
+        }
     }
 }
 
 # sorts fetch_urls in order of ping time
-proc sortsites {args} {
-    global fetch_urls fallback_mirror_site
+proc portfetch::sortsites {urls} {
+    global fallback_mirror_site master_sites
+    upvar $urls fetch_urls
+    variable urlmap
 
     set fallback_mirror_list [mirror_sites $fallback_mirror_site {} {}]
 
     foreach {url_var distfile} $fetch_urls {
-        global portfetch::$url_var
-        if {![info exists $url_var]} {
+        if {![info exists urlmap($url_var)]} {
             ui_error [format [msgcat::mc "No defined site for tag: %s, using master_sites"] $url_var]
-            set url_var master_sites
-            global portfetch::$url_var
+            set urlmap($url_var) $master_sites
         }
-        set urllist [set $url_var]
+        set urllist $urlmap($url_var)
         set hosts {}
         set hostregex {[a-zA-Z]+://([a-zA-Z0-9\.\-_]+)}
 
@@ -396,7 +453,7 @@ proc sortsites {args} {
             }
             ui_debug "$host ping time is $pingtimes($host)"
         }
-        
+
         set pinglist {}
         foreach site $urllist {
             regexp $hostregex $site -> host
@@ -405,123 +462,95 @@ proc sortsites {args} {
 
         set pinglist [ lsort -real -index 1 $pinglist ]
 
-        set $url_var {}
+        set urlmap($url_var) {}
         foreach pair $pinglist {
-            lappend $url_var [lindex $pair 0]
+            lappend urlmap($url_var) [lindex $pair 0]
         }
     }
 }
 
 # Perform the full checksites/checkpatchfiles/checkdistfiles sequence.
 # This method is used by distcheck target.
-proc checkfiles {args} {
-	# Set fetch_urls to be empty in case there is no file to fetch.
-	global fetch_urls
-	set fetch_urls {}
-	checksites
-	checkpatchfiles
-	checkdistfiles
+proc portfetch::checkfiles {urls} {
+    upvar $urls fetch_urls
+
+    checksites
+    checkpatchfiles fetch_urls
+    checkdistfiles fetch_urls
 }
 
 
 # Perform a CVS login and fetch, storing the CVS login
 # information in a custom .cvspass file
-proc cvsfetch {args} {
-    global workpath cvs.env cvs.cmd cvs.args cvs.post_args 
-    global cvs.root cvs.date cvs.tag cvs.password
+proc portfetch::cvsfetch {args} {
+    global workpath cvs.env cvs.cmd cvs.args cvs.post_args
+    global cvs.root cvs.date cvs.tag cvs.method cvs.password
     global patch_sites patchfiles filespath
 
-    set cvs.args "co ${cvs.args}"
+    set cvs.args "${cvs.method} ${cvs.args}"
+    if {${cvs.method} == "export" && ![string length ${cvs.tag}] && ![string length ${cvs.date}]} {
+        set cvs.tag "HEAD"
+    }
     if {[string length ${cvs.tag}]} {
-	set cvs.args "${cvs.args} -r ${cvs.tag}"
+        set cvs.args "${cvs.args} -r ${cvs.tag}"
     }
 
     if {[string length ${cvs.date}]} {
-	set cvs.args "${cvs.args} -D ${cvs.date}"
+        set cvs.args "${cvs.args} -D ${cvs.date}"
     }
 
     if {[regexp ^:pserver: ${cvs.root}]} {
-	set savecmd ${cvs.cmd}
-	set saveargs ${cvs.args}
-	set savepost_args ${cvs.post_args}
-	set cvs.cmd "echo ${cvs.password} | $portutil::autoconf::cvs_path"
-	set cvs.args login
-	set cvs.post_args ""
-	if {[catch {command_exec cvs -notty "" "2>&1"} result]} {
-	    return -code error [msgcat::mc "CVS login failed"]
-	}
-	set cvs.cmd ${savecmd}
-	set cvs.args ${saveargs}
-	set cvs.post_args ${savepost_args}
+        set savecmd ${cvs.cmd}
+        set saveargs ${cvs.args}
+        set savepost_args ${cvs.post_args}
+        set cvs.cmd "echo ${cvs.password} | ${cvs.cmd}"
+        set cvs.args login
+        set cvs.post_args ""
+        if {[catch {command_exec cvs -notty "" "2>&1"} result]} {
+            return -code error [msgcat::mc "CVS login failed"]
+        }
+        set cvs.cmd ${savecmd}
+        set cvs.args ${saveargs}
+        set cvs.post_args ${savepost_args}
     } else {
-	set env(CVS_RSH) ssh
+        set env(CVS_RSH) ssh
     }
 
     if {[catch {command_exec cvs "" "2>&1"} result]} {
-	return -code error [msgcat::mc "CVS check out failed"]
+        return -code error [msgcat::mc "CVS check out failed"]
     }
 
     if {[info exists patchfiles]} {
-	return [fetchfiles]
+        return [portfetch::fetchfiles]
     }
     return 0
 }
 
 # Perform an svn fetch
-proc svnfetch {args} {
-    global workpath prefix_frozen
-    global svn.env svn.cmd svn.args svn.post_args svn.tag svn.url
-    
-    # Look for the svn command, either in the path or in the prefix
-    set goodcmd 0
-    foreach svncmd "${svn.cmd} ${prefix_frozen}/bin/svn svn" {
- 	if { [file executable ${svncmd}] } {
- 	   	  set svn.cmd $svncmd
- 	   	  set goodcmd 1
- 	      break;
- 	   }
-    }
-    if { !$goodcmd } {
-    	ui_error "The subversion tool (svn) is required to fetch ${svn.url}."
-    	ui_error "Please install the subversion port before proceeding."
-		return -code error [msgcat::mc "Subversion check out failed"]
-    }
-    
-    set svn.args "checkout ${svn.args}"
-    if {[string length ${svn.tag}]} {
-		set svn.args "${svn.args} -r ${svn.tag}"
+proc portfetch::svnfetch {args} {
+    global svn.args svn.revision svn.method
+
+    set svn.args "${svn.method} ${svn.args}"
+    if {[string length ${svn.revision}]} {
+        set svn.args "${svn.args} -r ${svn.revision}"
     }
 
     if {[catch {command_exec svn "" "2>&1"} result]} {
-		return -code error [msgcat::mc "Subversion check out failed"]
+        return -code error [msgcat::mc "Subversion check out failed"]
     }
 
     if {[info exists patchfiles]} {
-	return [fetchfiles]
+        return [portfetch::fetchfiles]
     }
 
     return 0
 }
 
 # Perform a git fetch
-proc gitfetch {args} {
-    global worksrcpath prefix_frozen
-    global git.url git.branch git.sha1
-    
-    # Look for the git command
-    set git.cmd {}
-    foreach gitcmd "$portutil::autoconf::git_path $prefix_frozen/bin/git git" {
-        if {[file executable $gitcmd]} {
-            set git.cmd $gitcmd
-            break
-        }
-    }
-    if {${git.cmd} == {}} {
-        ui_error "git is required to fetch ${git.url}"
-        ui_error "Please install the git-core port before proceeding."
-        return -code error [msgcat::mc "Git command not found"]
-    }
-    
+proc portfetch::gitfetch {args} {
+    global worksrcpath
+    global git.url git.branch git.sha1 git.cmd
+
     set options "-q"
     if {[string length ${git.branch}] == 0} {
         # if we're just using HEAD, we can make a shallow repo
@@ -532,7 +561,7 @@ proc gitfetch {args} {
     if {[catch {system $cmdstring} result]} {
         return -code error [msgcat::mc "Git clone failed"]
     }
-    
+
     if {[string length ${git.branch}] > 0} {
         set env "GIT_DIR=${worksrcpath}/.git GIT_WORK_TREE=${worksrcpath}"
         set cmdstring "$env ${git.cmd} checkout -q ${git.branch} 2>&1"
@@ -541,32 +570,18 @@ proc gitfetch {args} {
             return -code error [msgcat::mc "Git checkout failed"]
         }
     }
-    
+
     if {[info exists patchfiles]} {
-        return [fetchfiles]
+        return [portfetch::fetchfiles]
     }
-    
+
     return 0
 }
 
 # Perform a mercurial fetch.
-proc hgfetch {args} {
+proc portfetch::hgfetch {args} {
     global worksrcpath prefix_frozen
-    global hg.url hg.tag
-
-    # Look for the hg command.
-    set hg.cmd {}
-    foreach hgcmd "$prefix_frozen/bin/hg hg" {
-        if {[file executable $hgcmd]} {
-            set hg.cmd $hgcmd
-            break
-        }
-    }
-    if {${hg.cmd} == {}} {
-        ui_error "hg is required to fetch ${hg.url}"
-        ui_error "Please install the mercurial port before proceeding."
-        return -code error [msgcat::mc "Mercurial command not found"]
-    }
+    global hg.url hg.tag hg.cmd
 
     set cmdstring "${hg.cmd} clone --rev ${hg.tag} ${hg.url} ${worksrcpath} 2>&1"
     ui_debug "Executing: $cmdstring"
@@ -575,7 +590,7 @@ proc hgfetch {args} {
     }
 
     if {[info exists patchfiles]} {
-        return [fetchfiles]
+        return [portfetch::fetchfiles]
     }
 
     return 0
@@ -583,149 +598,160 @@ proc hgfetch {args} {
 
 # Perform a standard fetch, assembling fetch urls from
 # the listed url variable and associated distfile
-proc fetchfiles {args} {
-	global distpath all_dist_files UI_PREFIX fetch_urls
-	global fetch.user fetch.password fetch.use_epsv fetch.ignore_sslcert fetch.remote_time
-	global distfile site
-	global portverbose
+proc portfetch::fetchfiles {args} {
+    global distpath all_dist_files UI_PREFIX
+    global fetch.user fetch.password fetch.use_epsv fetch.ignore_sslcert fetch.remote_time
+    global distfile site
+    global portverbose
+    variable fetch_urls
+    variable urlmap
 
-	if {![file isdirectory $distpath]} {
-		if {[catch {file mkdir $distpath} result]} {
-			return -code error [format [msgcat::mc "Unable to create distribution files path: %s"] $result]
-		}
-	}
-	
-	set fetch_options {}
-	if {[string length ${fetch.user}] || [string length ${fetch.password}]} {
-		lappend fetch_options -u
-		lappend fetch_options "${fetch.user}:${fetch.password}"
-	}
-	if {${fetch.use_epsv} != "yes"} {
-		lappend fetch_options "--disable-epsv"
-	}
-	if {${fetch.ignore_sslcert} != "no"} {
-		lappend fetch_options "--ignore-ssl-cert"
-	}
-	if {${fetch.remote_time} != "no"} {
-		lappend fetch_options "--remote-time"
-	}
-	if {$portverbose == "yes"} {
-		lappend fetch_options "-v"
-	}
-	set sorted no
-	
-	foreach {url_var distfile} $fetch_urls {
-		if {![file isfile $distpath/$distfile]} {
-			ui_info "$UI_PREFIX [format [msgcat::mc "%s doesn't seem to exist in %s"] $distfile $distpath]"
-			if {![file writable $distpath]} {
-				return -code error [format [msgcat::mc "%s must be writable"] $distpath]
-			}
-			if {!$sorted} {
-			    sortsites
-			    set sorted yes
-			}
-			global portfetch::$url_var
-			if {![info exists $url_var]} {
-				ui_error [format [msgcat::mc "No defined site for tag: %s, using master_sites"] $url_var]
-				set url_var master_sites
-				global portfetch::$url_var
-			}
-			unset -nocomplain fetched
-			foreach site [set $url_var] {
-				ui_msg "$UI_PREFIX [format [msgcat::mc "Attempting to fetch %s from %s"] $distfile $site]"
-				set file_url [portfetch::assemble_url $site $distfile]
-				set effectiveURL ""
-				if {![catch {eval curl fetch --effective-url effectiveURL $fetch_options {$file_url} ${distpath}/${distfile}.TMP} result] &&
-					![catch {system "mv ${distpath}/${distfile}.TMP ${distpath}/${distfile}"}]} {
+    if {![file isdirectory $distpath]} {
+        if {[catch {file mkdir $distpath} result]} {
+            elevateToRoot "fetch"
+            set elevated yes
+            if {[catch {file mkdir $distpath} result]} {
+                return -code error [format [msgcat::mc "Unable to create distribution files path: %s"] $result]
+            }
+        }
+    }
+    chownAsRoot $distpath
+    if {[info exists elevated] && $elevated == yes} {
+        dropPrivileges
+    }
 
-					# Special hack to check for sourceforge mirrors, which don't return a proper error code on failure
-					if {![string equal $effectiveURL $file_url] &&
-						[string match "*sourceforge*" $file_url] &&
-						[string match "*failedmirror*" $effectiveURL]} {
-						
-						# *SourceForge hackage in effect*
-						# The url seen by curl seems to have been a redirect to the sourceforge mirror page
-						ui_debug "[msgcat::mc "Fetching from sourceforge mirror failed"]"
-						exec rm -f ${distpath}/${distfile}.TMP
-						
-						# Continue on to try the next mirror, if any
-					} else {
-					
-						# Successful fetch
-						set fetched 1
-						break
-					
-					}
+    set fetch_options {}
+    if {[string length ${fetch.user}] || [string length ${fetch.password}]} {
+        lappend fetch_options -u
+        lappend fetch_options "${fetch.user}:${fetch.password}"
+    }
+    if {${fetch.use_epsv} != "yes"} {
+        lappend fetch_options "--disable-epsv"
+    }
+    if {${fetch.ignore_sslcert} != "no"} {
+        lappend fetch_options "--ignore-ssl-cert"
+    }
+    if {${fetch.remote_time} != "no"} {
+        lappend fetch_options "--remote-time"
+    }
+    if {$portverbose == "yes"} {
+        lappend fetch_options "-v"
+    }
+    set sorted no
 
-				} else {
-					ui_debug "[msgcat::mc "Fetching failed:"]: $result"
-					exec rm -f ${distpath}/${distfile}.TMP
-				}
-			}
-			if {![info exists fetched]} {
-				return -code error [msgcat::mc "fetch failed"]
-			}
-		}
-	}
+    foreach {url_var distfile} $fetch_urls {
+        if {![file isfile $distpath/$distfile]} {
+            ui_info "$UI_PREFIX [format [msgcat::mc "%s doesn't seem to exist in %s"] $distfile $distpath]"
+            if {![file writable $distpath]} {
+                return -code error [format [msgcat::mc "%s must be writable"] $distpath]
+            }
+            if {!$sorted} {
+                sortsites fetch_urls
+                set sorted yes
+            }
+            if {![info exists urlmap($url_var)]} {
+                ui_error [format [msgcat::mc "No defined site for tag: %s, using master_sites"] $url_var]
+                set urlmap($url_var) $master_sites
+            }
+            unset -nocomplain fetched
+            foreach site $urlmap($url_var) {
+                ui_msg "$UI_PREFIX [format [msgcat::mc "Attempting to fetch %s from %s"] $distfile $site]"
+                set file_url [portfetch::assemble_url $site $distfile]
+                set effectiveURL ""
+                if {![catch {eval curl fetch --effective-url effectiveURL $fetch_options {$file_url} ${distpath}/${distfile}.TMP} result] &&
+                    ![catch {file rename -force "${distpath}/${distfile}.TMP" "${distpath}/${distfile}"} result]} {
+
+                    # Special hack to check for sourceforge mirrors, which don't return a proper error code on failure
+                    if {![string equal $effectiveURL $file_url] &&
+                        [string match "http://*sourceforge.net/*" $file_url] &&
+                        [string match "http://*sourceforge.net/projects/*/files/" $effectiveURL]} {
+
+                        # *SourceForge hackage in effect*
+                        # The url seen by curl seems to have been a redirect to the sourceforge mirror page
+                        ui_debug "[msgcat::mc "Fetching from sourceforge mirror failed"]"
+                        file delete -force "${distpath}/${distfile}.TMP"
+
+                        # Continue on to try the next mirror, if any
+                    } else {
+
+                        # Successful fetch
+                        set fetched 1
+                        break
+
+                    }
+
+                } else {
+                    ui_debug "[msgcat::mc "Fetching failed:"]: $result"
+                    file delete -force "${distpath}/${distfile}.TMP"
+                }
+            }
+            if {![info exists fetched]} {
+                return -code error [msgcat::mc "fetch failed"]
+            }
+        }
+    }
     return 0
 }
 
 # Utility function to delete fetched files.
-proc fetch_deletefiles {args} {
-	global distpath fetch_urls
-	foreach {url_var distfile} $fetch_urls {
-		if {[file isfile $distpath/$distfile]} {
-			exec rm -f ${distpath}/${distfile}
-		}
-	}
+proc portfetch::fetch_deletefiles {args} {
+    global distpath
+    variable fetch_urls
+    foreach {url_var distfile} $fetch_urls {
+        if {[file isfile $distpath/$distfile]} {
+            file delete -force "${distpath}/${distfile}"
+        }
+    }
 }
 
 # Utility function to add files to a list of fetched files.
-proc fetch_addfilestomap {filemapname} {
-	global distpath fetch_urls $filemapname
-	foreach {url_var distfile} $fetch_urls {
-		if {[file isfile $distpath/$distfile]} {
-			filemap set $filemapname $distpath/$distfile 1
-		}
-	}
+proc portfetch::fetch_addfilestomap {filemapname} {
+    global distpath $filemapname
+    variable fetch_urls
+    foreach {url_var distfile} $fetch_urls {
+        if {[file isfile $distpath/$distfile]} {
+            filemap set $filemapname $distpath/$distfile 1
+        }
+    }
 }
 
 # Initialize fetch target and call checkfiles.
-proc fetch_init {args} {
+proc portfetch::fetch_init {args} {
     global distfiles distname distpath all_dist_files dist_subdir fetch.type fetch_init_done
-    
+    variable fetch_urls
+
     if {[info exists distpath] && [info exists dist_subdir] && ![info exists fetch_init_done]} {
-	    set distpath ${distpath}/${dist_subdir}
-	    set fetch_init_done yes
+        set distpath ${distpath}/${dist_subdir}
+        set fetch_init_done yes
     }
-    checkfiles
+    portfetch::checkfiles fetch_urls
 }
 
-proc fetch_start {args} {
-    global UI_PREFIX portname
-    
-    ui_msg "$UI_PREFIX [format [msgcat::mc "Fetching %s"] $portname]"
+proc portfetch::fetch_start {args} {
+    global UI_PREFIX name
+
+    ui_msg "$UI_PREFIX [format [msgcat::mc "Fetching %s"] $name]"
 }
 
 # Main fetch routine
 # If all_dist_files is not populated and $fetch.type == standard, then
 # there are no files to download. Otherwise, either do a cvs checkout
 # or call the standard fetchfiles procedure
-proc fetch_main {args} {
+proc portfetch::fetch_main {args} {
     global distname distpath all_dist_files fetch.type
 
     # Check for files, download if necessary
     if {![info exists all_dist_files] && "${fetch.type}" == "standard"} {
         return 0
     }
-    
+
     # Fetch the files
     switch -- "${fetch.type}" {
-    	cvs		{ return [cvsfetch] }
-    	svn		{ return [svnfetch] }
-    	git		{ return [gitfetch] }
-    	hg		{ return [hgfetch] }
-    	standard -
-    	default	{ return [fetchfiles] }
+        cvs     { return [cvsfetch] }
+        svn     { return [svnfetch] }
+        git     { return [gitfetch] }
+        hg      { return [hgfetch] }
+        standard -
+        default { return [portfetch::fetchfiles] }
     }
 }
