@@ -1,6 +1,6 @@
 # et:ts=4
 # portsubmit.tcl
-# $Id: portload.tcl 49087 2009-04-03 11:34:32Z raimue@macports.org $
+# $Id: portload.tcl 62219 2010-01-01 09:44:27Z jmr@macports.org $
 #
 # Copyright (c) 2007 MacPorts Project
 # Copyright (c) 2007 James D. Berry
@@ -37,11 +37,14 @@ package require portutil 1.0
 set org.macports.load [target_new org.macports.load portload::load_main]
 target_runtype ${org.macports.load} always
 target_provides ${org.macports.load} load 
-target_requires ${org.macports.load} portload::main
+target_requires ${org.macports.load} main
 
 namespace eval portload {
 }
 
+options load.asroot
+default load.asroot yes
+
 set_ui_prefix
 
 proc portload::load_main {args} {
