@@ -1,7 +1,7 @@
 # -*- coding: utf-8; mode: tcl; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- vim:fenc=utf-8:filetype=tcl:et:sw=4:ts=4:sts=4
 # portstartupitem.tcl
 #
-# $Id: portstartupitem.tcl 39012 2008-08-06 06:56:11Z jmr@macports.org $
+# $Id: portstartupitem.tcl 51521 2009-05-27 08:48:05Z jmr@macports.org $
 #
 # Copyright (c) 2004-2007 MacPorts Project
 # Copyright (c) 2006-2007 James D. Berry
@@ -68,10 +68,13 @@
 package provide portstartupitem 1.0
 package require portutil 1.0
 
+namespace eval portstartupitem {
+}
+
 set_ui_prefix
 
-proc startupitem_create_rcng {args} {
-    global prefix destroot portname os.platform
+proc portstartupitem::startupitem_create_rcng {args} {
+    global prefix destroot name os.platform
     global startupitem.name startupitem.requires
     global startupitem.start startupitem.stop startupitem.restart
     global startupitem.type
@@ -116,8 +119,8 @@ proc startupitem_create_rcng {args} {
     close ${fd}
 }
 
-proc startupitem_create_darwin_systemstarter {args} {
-    global UI_PREFIX prefix destroot destroot.keepdirs  portname os.platform
+proc portstartupitem::startupitem_create_darwin_systemstarter {args} {
+    global UI_PREFIX prefix destroot destroot.keepdirs name os.platform
     global startupitem.name startupitem.requires startupitem.init
     global startupitem.start startupitem.stop startupitem.restart startupitem.executable
     global startupitem.pidfile startupitem.logfile startupitem.logevents
@@ -174,10 +177,10 @@ proc startupitem_create_darwin_systemstarter {args} {
         # An executable is specified, and there is no init, start, stop, or restart
     } else {
         if { ![llength ${startupitem.start} ] } {
-            set startupitem.start [list "sh ${scriptdir}/${portname}.sh start"]
+            set startupitem.start [list "sh ${scriptdir}/${name}.sh start"]
         }
         if { ![llength ${startupitem.stop} ] } {
-            set startupitem.stop [list "sh ${scriptdir}/${portname}.sh stop"]
+            set startupitem.stop [list "sh ${scriptdir}/${name}.sh stop"]
         }
     }
     if { ![llength ${startupitem.requires} ] } {
@@ -374,7 +377,7 @@ RunService "$1"
     # Emit some information for the user
     ui_msg "###########################################################"
     ui_msg "# A startup item has been generated that will aid in"
-    ui_msg "# starting ${portname} with SystemStarter. It is disabled"
+    ui_msg "# starting ${name} with SystemStarter. It is disabled"
     ui_msg "# by default. Add the following line to /etc/hostconfig"
     ui_msg "# or ${prefix}/etc/rc.conf to start it at startup:"
     ui_msg "#"
@@ -382,8 +385,8 @@ RunService "$1"
     ui_msg "###########################################################"
 }
 
-proc startupitem_create_darwin_launchd {args} {
-    global UI_PREFIX prefix destroot destroot.keepdirs portname os.platform
+proc portstartupitem::startupitem_create_darwin_launchd {args} {
+    global UI_PREFIX prefix destroot destroot.keepdirs name os.platform
     global startupitem.name startupitem.uniquename startupitem.plist startupitem.location
     global startupitem.init startupitem.start startupitem.stop startupitem.restart startupitem.executable
     global startupitem.pidfile startupitem.logfile startupitem.logevents startupitem.netchange
@@ -422,10 +425,10 @@ proc startupitem_create_darwin_launchd {args} {
         set wrapper         "${itemdir}/${wrappername}"
 
         if { ![llength ${startupitem.start}] } {
-            set startupitem.start [list "sh ${scriptdir}/${portname}.sh start"]
+            set startupitem.start [list "sh ${scriptdir}/${name}.sh start"]
         }
         if { ![llength ${startupitem.stop}] } {
-            set startupitem.stop [list "sh ${scriptdir}/${portname}.sh stop"]
+            set startupitem.stop [list "sh ${scriptdir}/${name}.sh stop"]
         }
         if { ![llength ${startupitem.restart}] } {
             set startupitem.restart [list Stop Start]
@@ -584,7 +587,7 @@ proc startupitem_create_darwin_launchd {args} {
 
     # Make a symlink to the plist file
     file mkdir "${destroot}/Library/${daemondest}"
-    system "cd ${destroot}/Library/${daemondest} && ln -sf ${itemdir}/${plistname}"
+    ln -sf "${itemdir}/${plistname}" "${destroot}/Library/${daemondest}"
     
     # If launchd is not available, warn the user
     set haveLaunchd ${portutil::autoconf::have_launchd}
@@ -600,7 +603,7 @@ proc startupitem_create_darwin_launchd {args} {
     # Emit some information for the user
     ui_msg "###########################################################"
     ui_msg "# A startup item has been generated that will aid in"
-    ui_msg "# starting ${portname} with launchd. It is disabled"
+    ui_msg "# starting ${name} with launchd. It is disabled"
     ui_msg "# by default. Execute the following command to start it,"
     ui_msg "# and to cause it to launch at startup:"
     ui_msg "#"
@@ -608,7 +611,7 @@ proc startupitem_create_darwin_launchd {args} {
     ui_msg "###########################################################"
 }
 
-proc startupitem_create {args} {
+proc portstartupitem::startupitem_create {args} {
     global UI_PREFIX
     global startupitem.type os.platform
     
