@@ -1,6 +1,6 @@
 # et:ts=4
 # portuninstall.tcl
-# $Id: portuninstall.tcl 37349 2008-06-04 08:32:15Z jmr@macports.org $
+# $Id: portuninstall.tcl 62170 2009-12-30 14:03:29Z jmr@macports.org $
 #
 # Copyright (c) 2002 - 2003 Apple Computer, Inc.
 # All rights reserved.
@@ -92,12 +92,8 @@ proc uninstall {portname {v ""} optionslist} {
 		foreach dep $deplist { 
 			set depport [lindex $dep 2]
 			ui_debug "$depport depends on this port"
-			# xxx: Should look at making registry::installed return 0 or 
-			# something instead  of erroring.
-			if { ![catch {set installed [registry::installed $depport]} res] } {
-				if { [llength installed] > 0 } {
-					lappend dl $depport
-				}
+			if {[registry::entry_exists_for_name $depport]} {
+				lappend dl $depport
 			}
 		}
 		# Now see if we need to error
@@ -105,11 +101,8 @@ proc uninstall {portname {v ""} optionslist} {
 			if {[info exists options(ports_uninstall_follow-dependents)] && $options(ports_uninstall_follow-dependents) eq "yes"} {
 				foreach depport $dl {
 					# make sure it's still installed, since a previous dep uninstall may have removed it
-					# does registry::installed still error? A cursory look at the code says no, but above says yes
-					if { ![catch {set installed [registry::installed $depport]} res] } {
-						if { [llength installed] > 0 } {
-							portuninstall::uninstall $depport "" [array get options]
-						}
+					if {[registry::entry_exists_for_name $depport]} {
+						portuninstall::uninstall $depport "" [array get options]
 					}
 				}
 			} else {
@@ -131,10 +124,18 @@ proc uninstall {portname {v ""} optionslist} {
 
 	set installtype [registry::property_retrieve $ref installtype]
 	if { $installtype == "image" && [registry::property_retrieve $ref active] == 1} {
-		#return -code error [msgcat::mc "Registry Error: ${portname} ${version}_${revision}${variants} is active."]
-		portimage::deactivate $portname ${version}_${revision}${variants} $optionslist
+		if {[info exists options(ports_dryrun)] && $options(ports_dryrun) == "yes"} {
+			ui_msg "For $portname @${version}_${revision}${variants}: skipping deactivate (dry run)"
+		} else {
+			portimage::deactivate $portname ${version}_${revision}${variants} $optionslist
+		}
 	}
 
+	if {[info exists options(ports_dryrun)] && $options(ports_dryrun) == "yes"} {
+		ui_msg "For $portname @${version}_${revision}${variants}: skipping uninstall (dry run)"
+		return 0
+	}
+	
 	ui_msg "$UI_PREFIX [format [msgcat::mc "Uninstalling %s @%s_%s%s"] $portname $version $revision $variants]"
 
 	# Look to see if the port has registered an uninstall procedure
@@ -187,17 +188,21 @@ proc uninstall {portname {v ""} optionslist} {
 					}
 				}
 			}
-			# Normalize the file path to avoid removing the intermediate
-			# symlinks (remove the empty directories instead)
-			set theFile [compat filenormalize [lindex $f 0]]
-			lappend files $theFile
+			
+			set theFile [file normalize $fname]
+			if { [file exists $theFile] || (![catch {file type $theFile}] && [file type $theFile] == "link") } {
+			    # Normalize the file path to avoid removing the intermediate
+			    # symlinks (remove the empty directories instead)
+			    lappend files $theFile
 
-			# Split out the filename's subpaths and add them to the
-			# list as well.
-			set directory [file dirname $theFile]
-			while { [lsearch -exact $files $directory] == -1 } { 
-				lappend files $directory
-				set directory [file dirname $directory]
+			    # Split out the filename's subpaths and add them to the
+			    # list as well. The realpath call is necessary because file normalize
+			    # does not resolve symlinks on OS X < 10.6
+			    set directory [realpath [file dirname $theFile]]
+			    while { [lsearch -exact $files $directory] == -1 } { 
+				    lappend files $directory
+				    set directory [file dirname $directory]
+			    }
 			}
 		}
 
