@@ -1,9 +1,9 @@
 # et:ts=4
 # portimage.tcl
-# $Id$
+# $Id: portimage.tcl 58126 2009-09-22 17:29:57Z jmr@macports.org $
 #
 # Copyright (c) 2004 Will Barton <wbb4@opendarwin.org>
-# Copyright (c) 2002 Apple Computer, Inc.
+# Copyright (c) 2002 Apple Inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -14,7 +14,7 @@
 # 2. Redistributions in binary form must reproduce the above copyright
 #    notice, this list of conditions and the following disclaimer in the
 #    documentation and/or other materials provided with the distribution.
-# 3. Neither the name of Apple Computer, Inc. nor the names of its contributors
+# 3. Neither the name of Apple Inc. nor the names of its contributors
 #    may be used to endorse or promote products derived from this software
 #    without specific prior written permission.
 #
@@ -57,9 +57,6 @@ set UI_PREFIX "--> "
 # will also remove all of the references of the files from the registry's
 # file_map
 #
-# Compacting and Uncompacting of port images to save space will be implemented
-# at some point.
-#
 # For the creating and removing of links during activation and deactivation,
 # code very similar to what is used in portinstall is used.
 #
@@ -114,11 +111,6 @@ proc activate {name specifier optionslist} {
         if { [string equal [$requested state] "active"] } {
             return -code error "Image error: ${name} @${version}_${revision}${variants} is already active."
         }
-
-        # compaction is not yet supported
-        #if { [$requested compact] != 0 } {
-        #    return -code error "Image error: ${name} @${version}_${revision}${variants} is compacted."
-        #}
     }
 
     _activate_contents $port $force
@@ -166,11 +158,6 @@ proc deactivate {name spec optionslist} {
         return -code error "Image error: ${name} @${specifier} is not active."
     }
 
-    # compaction not yet supported
-    #if { [registry::property_retrieve $ref compact] != 0 } {
-    #    return -code error "Image error: ${name} @${specifier} is compacted."
-    #}
-
     registry::check_dependents $port $force
 
     set imagedir [$requested imagedir]
@@ -180,18 +167,6 @@ proc deactivate {name spec optionslist} {
     $requested state imaged
 }
 
-proc compact {name v} {
-    global UI_PREFIX
-
-    throw registry::image-error "Image error: compact/uncompact not yet implemented."
-}
-
-proc uncompact {name v} {
-    global UI_PREFIX
-
-    throw registry::image-error "Image error: compact/uncompact not yet implemented."
-}
-
 proc _check_registry {name specifier} {
     global UI_PREFIX
 
@@ -254,10 +229,10 @@ proc _activate_file {srcfile dstfile} {
             ui_debug "activating file: $dstfile"
             # Try a hard link first and if that fails, a symlink
             try {
-                compat filelinkhard $dstfile $srcfile
+                file link -hard $dstfile $srcfile
             } catch {*} {
                 ui_debug "hardlinking $srcfile to $dstfile failed; symlinking instead"
-                compat filelinksymbolic $dstfile $srcfile
+                file link -symbolic $dstfile $srcfile
             }
         }
         default {
@@ -420,12 +395,12 @@ proc _deactivate_contents {port force} {
             # instead of the paths we currently have, users' registry won't
             # match and activate will say that some file exists but doesn't
             # belong to any port.
-            set theFile [compat filenormalize $file]
+            set theFile [file normalize $file]
             lappend files $theFile
 
             # Split out the filename's subpaths and add them to the image list
             # as well.
-            set directory [file dirname $theFile]
+            set directory [realpath [file dirname $theFile]]
             while { [lsearch -exact $files $directory] == -1 } {
                 lappend files $directory
                 set directory [file dirname $directory]
